//+------------------------------------------------------------------+
//|                                                  SampleMoney.mqh |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
#include <Expert\ExpertMoney.mqh>
#include <Trade\DealInfo.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=                          |
//| Type=Money                                                       |
//| Name=Sample                                                      |
//| Class=CSampleMoney                                               |
//| Page=                                                            |
//| Parameter=Lots,double,0.1                                        |
//| Parameter=Percent,double,100.0                                   |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//|  CSampleMoney.                                              |
//| :                      |
//|                  .           |
//|                 CExpertMoney.         |
//+------------------------------------------------------------------+
class CSampleMoney : public CExpertMoney
  {
protected:
   //---  
   double            m_lots;                     //    "" 

public:
                     CSampleMoney();
   //---    
   void              Lots(double lots) { m_lots=lots; }
   //---    
   virtual bool      ValidationSettings();
   //---   
   virtual double    CheckOpenLong(double price,double sl);
   virtual double    CheckOpenShort(double price,double sl);

protected:
   double            CheckPrevLoss();
  };
//+------------------------------------------------------------------+
//|  CSampleMoney.                                        |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void CSampleMoney::CSampleMoney()
  {
//---   -
   m_lots=0.1;
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true-  ,  false.             |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CSampleMoney::ValidationSettings()
  {
//---    
   if(!CExpertMoney::ValidationSettings()) return(false);
//---  
   if(m_lots<m_symbol.LotsMin() || m_lots>m_symbol.LotsMax())
     {
      printf(__FUNCTION__+":        %f  %f",m_symbol.LotsMin(),m_symbol.LotsMax());
      return(false);
     }
   if(MathAbs(m_lots/m_symbol.LotsStep()-MathRound(m_lots/m_symbol.LotsStep()))>1.0E-10)
     {
      printf(__FUNCTION__+":      %f",m_symbol.LotsStep());
      return(false);
     }
//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|      .                 |
//| INPUT:  .                                                     |
//| OUTPUT: lot-if successful, 0.0 otherwise.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
double CSampleMoney::CheckOpenLong(double price,double sl)
  {
   if(m_symbol==NULL) return(0.0);
//---   
   double lot=2*CheckPrevLoss();
   if(lot==0.0) lot=m_lots;
//---  
   double maxvol=m_symbol.LotsMax();
   if(lot>maxvol) lot=maxvol;
//---   
   if(price==0.0) price=m_symbol.Ask();
   maxvol=m_account.MaxLotCheck(m_symbol.Name(),ORDER_TYPE_BUY,price,m_percent);
   if(lot>maxvol) lot=maxvol;
//---   
   return(lot);
  }
//+------------------------------------------------------------------+
//|      .                |
//| INPUT:  .                                                     |
//| OUTPUT: lot-if successful, 0.0 otherwise.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
double CSampleMoney::CheckOpenShort(double price,double sl)
  {
   if(m_symbol==NULL) return(0.0);
//---   
   double lot=2*CheckPrevLoss();
   if(lot==0.0) lot=m_lots;
//---  
   double maxvol=m_symbol.LotsMax();
   if(lot>maxvol) lot=maxvol;
//---   
   if(price==0.0) price=m_symbol.Bid();
   maxvol=m_account.MaxLotCheck(m_symbol.Name(),ORDER_TYPE_SELL,price,m_percent);
   if(lot>maxvol) lot=maxvol;
//---   
   return(lot);
  }
//+------------------------------------------------------------------+
//|    .                        |
//| INPUT:  .                                                     |
//| OUTPUT:   -  ,  0.0 |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
double CSampleMoney::CheckPrevLoss()
  {
   double lot=0.0;
//---     
   HistorySelect(0,TimeCurrent());
//--- 
   int       deals=HistoryDealsTotal();  //    
   CDealInfo deal;
//---   
   for(int i=deals-1;i>=0;i--)
     {
      if(!deal.SelectByIndex(i))
        {
         printf(__FUNCTION__+":     ");
         break;
        }
      //---  
      if(deal.Symbol()!=m_symbol.Name()) continue;
      //---  
      if(deal.Profit()<0.0) lot=deal.Volume();
      break;
     }
//---  
   return(lot);
  }
//+------------------------------------------------------------------+